#
# Aura Engine (https://gitlab.servus.at/aura/engine)
#
# Copyright (C) 2017-2020 - The Aura Engine Team.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from enum import Enum



class RedisChannel(Enum):
    STANDARD = "aura"

    DPE_REPLY = "delete_playlist_entry_reply"
    FNP_REPLY = "fetch_new_programme_reply"
    GAP_REPLY = "get_act_programme_reply"
    GS_REPLY  = "get_status_reply"
    GCS_REPLY = "get_connection_status_reply"
    GNF_REPLY = "get_next_file_reply"
    IPE_REPLY = "insert_playlist_entry_reply"
    IP_REPLY  = "init_player_reply"
    TS_REPLY  = "track_service_reply"
    MPE_REPLY = "move_playlist_entry_reply"
    PMQ_REPLY = "print_message_queue_reply"
    RDB_REPLY = "recreate_database_reply"
    SNF_REPLY = "get_next_file_reply"


class Channel(Enum):
    """
    Channel name mappings to the Liqidsoap channel IDs
    """
    FILESYSTEM_A = "in_filesystem_0"
    FILESYSTEM_B = "in_filesystem_1"
    HTTP_A  = "in_http_0"
    HTTP_B  = "in_http_1"
    HTTPS_A  = "in_https_0"
    HTTPS_B  = "in_https_1"
    LIVE_0 = "linein_0"
    LIVE_1 = "linein_1"
    LIVE_2 = "linein_2"
    LIVE_3 = "linein_3"
    LIVE_4 = "linein_4"

    def __str__(self):
        return str(self.value)


class ChannelType(Enum):
    """
    Engine channel types mapped to `Entry` source types.
    """
    FILESYSTEM = {
        "id": "fs",
        "channels": [Channel.FILESYSTEM_A, Channel.FILESYSTEM_B]
    }
    HTTP = {
        "id": "http",
        "channels": [Channel.HTTP_A, Channel.HTTP_B]
    }
    HTTPS = {
        "id": "https",
        "channels": [Channel.HTTPS_A, Channel.HTTPS_B]
    }
    LIVE = {
        "id": "live",
        "channels": [
            Channel.LIVE_0, 
            Channel.LIVE_1,
            Channel.LIVE_2,
            Channel.LIVE_3,
            Channel.LIVE_4
        ]
    }

    @property
    def channels(self):
        return self.value["channels"]

    def __str__(self):
        return str(self.value["id"])


class TransitionType(Enum):
    """
    Types of fade-in and fade-out transition.
    """
    INSTANT = "instant"
    FADE = "fade"


class PlaylistType(Enum):
    DEFAULT     = { "id": 0, "name": "default" }    # Default play mode
    SHOW        = { "id": 1, "name": "show" }       # The first played when some default playlist fails
    TIMESLOT    = { "id": 2, "name": "timeslot" }   # The second played when the timeslot fallback fails
    STATION     = { "id": 3, "name": "station" }    # The last played when everything else fails

    @property
    def id(self):
        return self.value["id"]

    def __str__(self):
        return str(self.value["name"])


class TimerType(Enum):
    SWITCH = "switch"
    FADEIN = "fadein"
    FADEOUT = "fadeout"


class EntryQueueState(Enum):
    OKAY = "ok"
    CUT = "cut"
    OUT_OF_SCHEDULE = "oos"

class EntryPlayState(Enum):
    UNKNOWN = "unknown"
    LOADING = "loading"
    READY = "ready_to_play"
    PLAYING = "playing"
    FINISHED = "finished"

class LiquidsoapResponse(Enum):
    SUCCESS = "Done"
    STREAM_STATUS_POLLING = "polling"
    STREAM_STATUS_STOPPED = "stopped"
    STREAM_STATUS_CONNECTED = "connected"
    