#
# Aura Engine (https://gitlab.servus.at/aura/engine)
#
# Copyright (C) 2017-2020 - The Aura Engine Team.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import time
import logging
import datetime

from modules.communication.redis.statestore import RedisStateStore
from modules.communication.mail import AuraMailer
from modules.base.exceptions import PlaylistException
from modules.base.enum import RedisChannel
from modules.base.logger import AuraLogger

"""
Send and receive redis messages
"""


# ------------------------------------------------------------------------------------------ #
class RedisMessenger():
    logger = None
    rstore = None

    # ------------------------------------------------------------------------------------------ #
    def __init__(self, config):
        super(RedisMessenger, self).__init__()
        """
            Constructor
        """
        self.logger = logging.getLogger("AuraEngine")
        self.channel = RedisChannel.STANDARD
        self.section = ''
        self.rstore = RedisStateStore(config)
        self.errnr = '00'
        self.components = {'controller':'01', 'scheduling':'02', 'playd':'03', 'recorder':'04', 'helpers':'09'}
        self.fromMail = ''
        self.adminMails = ''

    # ------------------------------------------------------------------------------------------ #
    def set_channel(self, channel):
        """
        Einen "Kanal" setzen - zb scheduling
        @type channel: string
        @param channel: Kanal/Name der Komponente
        """
        self.channel = channel
        if channel in self.components:
            self.errnr = self.components[channel]
        self.rstore.set_channel(channel)

    # ------------------------------------------------------------------------------------------ #
    def set_section(self, section):
        """
        Einen Sektion / Gültigkeitsbereich der Meldung setzen - zb internal
        @type section: string
        @param section: Gültigkeitsbereich
        """
        self.section = section


#     # ------------------------------------------------------------------------------------------ #
#     def set_mail_addresses(self, fromMail, adminMails):
#         """
#         Einen Sektion / Gültigkeitsbereich der Meldung setzen - zb internal
#         @type section: string
#         @param section: Gültigkeitsbereich
#         """
#         self.fromMail = fromMail
#         self.adminMails = adminMails

#     # ------------------------------------------------------------------------------------------ #
#     def send(self, message, code, level, job, value='', section=''):
#         """
#         Eine Message senden
#         @type message:  string
#         @param message: menschenverständliche Nachricht
#         @type code:     string
#         @param code:    Fehlercode - endet mit 00 bei Erfolg
#         @type level:    string
#         @param level:   Error-Level - info, warning, error, fatal
#         @type job:      string
#         @param job:     Name der ausgeführten Funktion
#         @type value:    string
#         @param value:   Ein Wert
#         @type section:  string
#         @param section: Globale Sektion überschreiben
#         """
#         section = self.section if section == '' else section
#         self.time = str(datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S:%f'))
#         self.utime = time.time()
#         state = {'message':message.strip().replace("'","\\'"), 'code':self.errnr + str(code),'job':job,'value':value}
#         self.rstore.set_section(section)
#         self.rstore.store(level, state)

#         if level == 'info' or level == 'success':
#             self.logger.info(message)
#         elif level == 'warning':
#             self.logger.warning(message)
#         elif level == 'error':
#             self.logger.error(message)
#             self.send_admin_mail(level, message, state)

#         elif level == 'fatal':
#             self.logger.critical(message)
#             self.send_admin_mail(level, message, state)

#     # ------------------------------------------------------------------------------------------ #
#     def say_alive(self):
#         """
#         Soll alle 20 Sekunden von den Komponenten ausgeführt werden,
#         um zu melden, dass sie am Leben sind
#         """
#         self.rstore.set_alive_state()

#     # ------------------------------------------------------------------------------------------ #
#     def get_alive_state(self, channel):
#         """
#         Live State abfragen
#         @type channel: string
#         @param channel: Channel/Komponente
#         """
#         return self.rstore.get_alive_state(channel)

#     # ------------------------------------------------------------------------------------------ #
#     def set_state(self, name, value, expires=None, channel=None):
#         """
#         Kündigt einen Event an
#         @type name: string
#         @param name: Name des state
#         @type value: string
#         @param value: Wert
#         @type channel: string
#         @param channel: Kanal (optional)
#         """
#         if not channel:
#             channel = self.channel

#         self.rstore.set_state(name, value, expires, channel)

#     # ------------------------------------------------------------------------------------------ #
#     def queue_add_event(self, name, eventtime, value, channel=None):
#         """
#         Kündigt einen Event an
#         @type name: string
#         @param name: der Name des Events
#         @type eventtime: string|datetime.datetime
#         @param eventtime: Datum und Zeit des events
#         @type value: dict
#         @param value: Werte
#         @type channel: string
#         @param channel: Kanal (optional)
#         """
#         if not channel:
#             channel = self.channel

#         if type(eventtime) == type(str()):
#             eventtime_str = datetime.datetime.strptime(eventtime[0:16].replace(' ','T'), "%Y-%m-%dT%H:%M").strftime("%Y-%m-%dT%H:%M")

#         elif type(eventtime) is datetime.datetime:
#             eventtime_str = eventtime.strftime("%Y-%m-%dT%H:%M")

#         else:
#             raise TypeError('eventtime must be a datetime.date or a string, not a %s' % type(eventtime))

#         self.rstore.queue_add_event(eventtime_str, name, value, channel)

#     # ------------------------------------------------------------------------------------------ #
#     def queue_remove_events(self, name, channel=None):
#         """
#         Löscht Events
#         @type name: string
#         @param name: der Name des Events
#         @type channel: string
#         @param channel: Kanal (optional)
#         """
#         if not channel:
#             channel = self.channel

#         self.rstore.queue_remove_events(name, channel)

#     # ------------------------------------------------------------------------------------------ #
#     def fire_event(self, name, value, channel=None):
#         """
#         Feuert einen Event
#         @type name: string
#         @param name: der Name des Events
#         @type value: dict
#         @param value: Werte
#         @type channel: string
#         @param channel: Kanal (optional)
#         """
#         if not channel:
#             channel = self.channel

#         self.rstore.fire_event(name, value, channel)

#     # ------------------------------------------------------------------------------------------ #
#     def get_event_queue(self, name=None, channel=None):
#         """
#         Holt events eines Kanals
#         @type channel: string
#         @param channel: Kanal (optional)
#         @rtype: list
#         @return: Liste der Events
#         """
#         queue = self.rstore.get_event_queue(name, channel)
#         return queue

#     # ------------------------------------------------------------------------------------------ #
#     def get_events(self, name=None, channel=None):
#         """
#         Holt events eines Kanals
#         @type channel: string
#         @param channel: Kanal (optional)
#         @rtype: list
#         @return: Liste der Events
#         """
#         events = self.rstore.get_events(name, channel)
#         return events

#     # ------------------------------------------------------------------------------------------ #
#     def get_event(self, name=None, channel=None):
#         """
#         Holt event eines Kanals
#         @type channel: string
#         @param channel: Kanal (optional)
#         @rtype: dict
#         @return: Event
#         """
#         events = self.rstore.get_events(name, channel)
#         result = False
#         if events:
#             result = events.pop(0)
#         return result

#     # ------------------------------------------------------------------------------------------ #
#     def send_admin_mail(self, level, message, state):
#         """
#         Sendent mail an Admin(s),
#         @type message: string
#         @param message: Die Message
#         @type state: dict
#         @param state: Der State
#         @return result
#         """
        
#         # FIXME Make Mailer functional: Invalid constructor
#         if self.fromMail and self.adminMails:
#             #subject = "Possible comba problem on job " + state['job'] + " - " + level
#             mailmessage = "Hi Admin,\n comba reports a possible problem\n\n"
#             mailmessage = mailmessage + level + "!\n"
#             mailmessage = mailmessage + message + "\n\n"
#             mailmessage = mailmessage + "Additional information:\n"
#             mailmessage = mailmessage + "##################################################\n"
#             mailmessage = mailmessage + "Job:\t" + state['job'] + "\n"
#             mailmessage = mailmessage + "Code:\t" + state['code'] + "\n"
#             mailmessage = mailmessage + "Value:\t" + str(state['value']) + "\n"

#             #mailer = AuraMailer(self.adminMails, self.fromMail)
#             #mailer.send_admin_mail(subject, mailmessage)
#         else:
#             return False

#     # ------------------------------------------------------------------------------------------ #
#     def receive(self):
#         """
#         Bisher wird nichts empfangen
#         """
#         return ""

#     # ------------------------------------------------------------------------------------------ #
#     def get_next_file_for(self, playlisttype):
#         next = self.rstore.db.get('next_'+playlisttype+'file')

#         if next is None:
#             next = b""

#         return next.decode('utf-8')

    # ------------------------------------------------------------------------------------------ #
    # def on_play(self, info):
    #     result = self.rstore.db.get('on_play')

    #     if result is None:
    #         result = b""

    #     return result.decode('utf-8')

    # ------------------------------------------------------------------------------------------ #
    # def set_next_file_for(self, playlisttype, file):
    #     self.rstore.db.set("next_" + playlisttype + "file", file)


