#!/usr/bin/python3.7

#
# Aura Engine (https://gitlab.servus.at/aura/engine)
#
# Copyright (C) 2017-2020 - The Aura Engine Team.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import json
import logging
import sqlalchemy
import decimal

from modules.core.engine import SoundSystem
from modules.base.config import AuraConfig
from modules.database.model import Schedule, ScheduleEntry
from modules.scheduling.scheduler import AuraScheduler, AuraCalendarService

def alchemyencoder(obj):
    """JSON encoder function for SQLAlchemy special classes."""
    if isinstance(obj, datetime.date):
        return obj.isoformat()
    elif isinstance(obj, decimal.Decimal):
        return float(obj)
    elif isinstance(obj, sqlalchemy.orm.state.InstanceState):
        return ""
    #elif isinstance(obj, Schedule):
    #    return json.dumps([obj._asdict()], default=alchemyencoder)
    else:
        return str(obj)

#    programme_as_string = json.dumps([se[0]._asdict()], default=alchemyencoder)
#    print(programme_as_string)

def select_current_programme():
    # select_programme()
    config = AuraConfig()
    config.read_config()

    soundsystem = SoundSystem(config.config)
    sched = AuraScheduler(config.config)

    soundsystem.scheduler = sched
    sched.soundsystem = soundsystem

    programme = sched.load_programme_from_db()

    for show in programme:
        print(show)

def fadeout(lsc):
    entry = ScheduleEntry.select_programme()
    lsc.fade_out(entry, 2)

def fadein(lsc):
    entry = ScheduleEntry.select_programme()
    lsc.fade_in(entry, 1)

def fetch_new_programme():
    config = AuraConfig()
    config.read_config()
    acs = AuraCalendarService(config.config)


    queue = acs.get_queue()

    # start fetching thread
    acs.start()

    # wait for the end
    response = queue.get()

# # ## ## ## ## ## # #
# # ENTRY FUNCTION # #
# # ## ## ## ## ## # #
def main():
    fetch_new_programme()


# # ## ## ## ## ## ## # #
# # End ENTRY FUNCTION # #
# # ## ## ## ## ## ## # #


if __name__ == "__main__":
    main()

